/***************************************************************************
*
*   Module:             diags
*
*   Filename:           sysevent.h
*
*   Author:             Brian Tran
*
*   Created:            09/03/2002
*
*   Description:
*
*   For system event interrupt
*
****************************************************************************
*                 Source Control System Information
*
*    $Id: 120sc_sysevent.h,v 1.7 2004/08/03 20:31:18 ted Exp $
*
****************************************************************************
*                       Modification History
*
*    $Log: 120sc_sysevent.h,v $
*    Revision 1.7  2004/08/03 20:31:18  ted
*    Add support for Anafa2 as a level-triggered acvive low interrupt.
*    Clean up some formatting.
*
*    Revision 1.6  2004/03/02 21:48:03  brian
*    Bugzilla number:
*    FIx a bug in defining the codes for sysevent write (overlapping)
*
*    Revision 1.5  2004/02/20 04:42:44  brian
*    Bugzilla number:
*    Added hardware error interrupt
*
*    Revision 1.4  2004/02/12 04:46:04  brian
*    Bugzilla number:
*    Support system event in 120
*
*    Revision 1.3  2003/12/18 05:12:42  brian
*    Bugzilla number:
*    Add i2c debug hook
*
*    Revision 1.2  2003/10/30 23:08:57  brian
*    Bugzilla number:
*    Support seti2croute in 120 kernel
*
*    Revision 1.1  2003/10/29 01:16:16  brian
*    Bugzilla number:
*    For 120
*
*
***************************************************************************
*                       (c) COPYRIGHT 2000, 2001, 2002
*                           TopSpin Communications, Inc.
*                           Palo Alto, CA
**************************************************************************/


#ifndef _SYSEVENT_H
#define _SYSEVENT_H


#define SYSEVENT_MAJOR  250
#define SYSEVENT_MINOR  0

#define FPGA_REGADDR_BASE		0x1f0100000


/* UIC0 Interrupts */
#define ANAFA2_IRQ_NUM			23	/* IRQ0 */
#define SMM_HEALTHY_IRQ_NUM		24	/* IRQ1 */
#define OPTICAL_FAULT_IRQ_NUM		25	/* IRQ2 */
#define PS_FAIL_IRQ_NUM			28	/* IRQ5 */
#define FPGA_IRQ_NUM			27	/* IRQ4 */
#define SMM_FAULT_IRQ_NUM		29	/* IRQ6 */

/* UIC1 Interrupts */
#define FAN_FAULT_IRQ_NUM		53	/* IRQ10 - UIC1 - UIC0 at non-critical int */



#define FPGA_INT_EN			0x11
#define PS_FAIL_INT_EN			0x12
#define IBML_INT_EN			0x13
#define LM_INT_EN			0x14
#define LOOP_ENABLE			0x15
#define I2CTTY_ENABLE			0x16
#define I2CTTY_DISABLE			0x17


#define FPGA_INT_DIS			0x21
#define PS_FAIL_INT_DIS			0x22
#define IBML_INT_DIS			0x23
#define LM_INT_DIS			0x24

#define LOOP_DISABLE			0x25

#define LOCAL_SWITCH_I2C_ADDR		0x70

/* define the interrupt bit positions */
/* UIC0 */
#define TS120_ANAFA2_INT_MASK		0x00000100		/* IRQ0 */
#define TS120_SMM_HEALTHY_INT_MASK	0x00000080		/* IRQ1 */
#define TS120_OPTICAL_FAULT_INT_MASK	0x00000040		/* IRQ2 */
#define TS120_FPGA_INT_MASK		0x00000010		/* IRQ4 */
#define TS120_PS_FAIL_INT_MASK		0x00000008		/* IRQ5 */
#define TS120_SMM_FAULT_INT_MASK	0x00000004		/* IRQ6 */
#define TS120_UIC0_EDGE_INT_MASK	(TS120_SMM_FAULT_INT_MASK    | \
					TS120_SMM_HEALTHY_INT_MASK   | \
					TS120_PS_FAIL_INT_MASK       | \
					TS120_OPTICAL_FAULT_INT_MASK | \
					TS120_FPGA_INT_MASK)
#define TS120_UIC0_LEVEL_INT_MASK	TS120_ANAFA2_INT_MASK

#define TS120_UIC0_INT_MASK		(TS120_UIC0_EDGE_INT_MASK    | \
					 TS120_UIC0_LEVEL_INT_MASK)

/* UIC1 */
#define TS120_FAN_INT_MASK		0x00000400		/* IRQ10 */

#endif _SYSEVENT_H
