#include <linux/config.h>
#include <linux/version.h>
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/init.h>
#include <asm/uaccess.h>
#include <asm/io.h>
#include <asm/machdep.h>
#include <asm/ocp.h>

#define DCR_MAJOR	251

extern unsigned int get_dcr(unsigned short dcrn);
extern unsigned int set_dcr(unsigned short dcrn, unsigned int value);


int dcr_init_module(void);
void dcr_cleanup_module(void);


/* Function Prototypes */
static ssize_t dcr_read(struct file *file, char *buf, size_t count, loff_t *offset);
static ssize_t dcr_write(struct file *file, const char *buf, size_t count, loff_t *offset);
static int dcr_ioctl (struct inode *inode, struct file *file, unsigned int cmd, unsigned long arg);


/* File Operation Table */
static struct file_operations char_fops = {
    owner: THIS_MODULE,
    read: dcr_read,
    write: dcr_write,
    ioctl: dcr_ioctl,
};



int dcr_init_module(void)
{

    /* register char device */
    if (register_chrdev(DCR_MAJOR, "dcrmodule", &char_fops)) {
        printk("Unable to register dcrmodule\n");
        return -EIO;
    }
    
    return 0;
}


void dcr_cleanup_module(void)
{

    unregister_chrdev(DCR_MAJOR, "dcrmodule");
}



/* Read routine */
static ssize_t dcr_read(struct file *file, char *buf, size_t count, loff_t *offset)
{
    unsigned int value;
    unsigned short dcraddr = (unsigned long) file->private_data; 

    value = get_dcr(dcraddr);
    copy_to_user(buf, &value, sizeof(int));

    return sizeof(int);

}


/* Write routine */
static ssize_t dcr_write(struct file *file, const char *buf, size_t count, loff_t *offset)
{
    unsigned int value;
    unsigned short dcraddr = (unsigned long) file->private_data;

    copy_from_user(&value, buf, sizeof(int));

    set_dcr(dcraddr, value);

    return sizeof(int);

}


/* Ioctl routine */
static int dcr_ioctl (struct inode *inode, struct file *file, unsigned int cmd, unsigned long arg)
{

    /* default with only 1 command for now */
    file->private_data = (void *)arg;

    return 0;
}
