/***************************************************************************
*
*   Module:
*
*   Filename:           ecc.c
*
*   Author:             Brian Tran
*
*   Created:            04/30/2005
*
*   Description:
*
*   Routines to support ECC checking and reporting
*
****************************************************************************
*                 Source Control System Information
*
*    $Id: ecc.c,v 1.1 2005/05/02 01:10:59 brian Exp $
*
****************************************************************************
*                       Modification History
*
*    $Log: ecc.c,v $
*    Revision 1.1  2005/05/02 01:10:59  brian
*    Bugzilla number:
*    Add function to report ECC errors
*
*
***************************************************************************
*                       (c) COPYRIGHT 2000, 2001, 2002
*                           TopSpin Communications, Inc.
*                           Palo Alto, CA
**************************************************************************/
#ifdef RCSID
static const char rcsid[] = "$Id: ecc.c,v 1.1 2005/05/02 01:10:59 brian Exp $";
#endif


#include <linux/config.h>
#include <linux/version.h>
#include <linux/module.h>
#include <linux/poll.h>
#include <linux/kernel.h>
#include <linux/proc_fs.h>
#include <linux/init.h>
#include <linux/blk.h>
#include <linux/unistd.h>
#include <asm/uaccess.h>
#include <asm/io.h>
#include <asm/irq.h>
#include <asm/machdep.h>
#include <asm/ocp.h>
#include <linux/delay.h>
#include <i2c-errcode.h>
#include <topspin_types.h>
#include <fpga.h>
#include <system_interrupt_code.h>
#include "ecc.h"

static int  ecc_count;


/* interrupt handler */
void ecc_ce_handler(int this_irq, void *dev_id, struct pt_regs *regs)
{
    int ecc_besr0;    
    int ecc_besr1;   
    int ecc_bear; 

    mtdcr(SDRAM0_CFGADDR, SDRAM0_BESR0);
    ecc_besr0 = mfdcr(SDRAM0_CFGDATA);
    mtdcr(SDRAM0_CFGDATA, ecc_besr0);

    mtdcr(SDRAM0_CFGADDR, SDRAM0_BESR1);
    ecc_besr1 = mfdcr(SDRAM0_CFGDATA);
    mtdcr(SDRAM0_CFGDATA, ecc_besr1);

    mtdcr(SDRAM0_CFGADDR, SDRAM0_BEAR);
    ecc_bear = mfdcr(SDRAM0_CFGDATA);
    mtdcr(SDRAM0_CFGDATA, ecc_bear);

    mtdcr(SDRAM0_CFGADDR, SDRAM0_ECCESR);
    mtdcr(SDRAM0_CFGDATA, -1);


    ecc_count ++;
}


int ecc_init(void)
{
    /* init global vars */
    ecc_count = 0;

    /* request interrupt */
    if (request_irq(ECC_CE_INT_NUM, ecc_ce_handler, SA_INTERRUPT, "ECC Correctable Error Intr", NULL)) {
        printk(KERN_ERR "ecc_init_module: Request IRQ %d Failed\n", ECC_CE_INT_NUM);
        return -1;
    }

    return 0;
}


void ecc_cleanup(void)
{
    free_irq(ECC_CE_INT_NUM, NULL);
}



/* Read routine
 * -----------
 * Return 4 if there is an ecc error. Return 0 othwrwise.
 */

ssize_t ecc_read(struct file *file, char *buf, size_t count, loff_t *offset)
{
    copy_to_user(buf, &ecc_count, sizeof(int));
    return sizeof(int);
}                           


