/***************************************************************************
*
*   Module:             
*
*   Filename:           fruerror.h
*
*   Author:             Brian Tran
*
*   Created:            04/25/2005
*
*   Description:
*  
*   Routines to support run-time errors
*
****************************************************************************
*                 Source Control System Information
*
*    $Id: fruerror.h,v 1.1 2005/04/27 09:32:50 brian Exp $
*
****************************************************************************
*                       Modification History
*
*    $Log: fruerror.h,v $
*    Revision 1.1  2005/04/27 09:32:50  brian
*    Bugzilla number:
*    Support runtime error reporting
*
*
***************************************************************************
*                       (c) COPYRIGHT 2000, 2001, 2002
*                           TopSpin Communications, Inc.
*                           Palo Alto, CA
**************************************************************************/
#ifdef RCSID
static const char rcsid[] = "$Id: fruerror.h,v 1.1 2005/04/27 09:32:50 brian Exp $";
#endif

#ifndef _FRUERROR_H
#define _FRUERROR_H

/*
 * init_fruerror
 * -------------
 * Init data structure for a FRU. Call only once during startup
 */
void init_fru_error(int fru_location, tRUNTIME_FRU_ERROR *fruerror);

/*
 * reset_fruerror
 * -------------
 * Reset data structure for a FRU. Clear all current errors.
 */
void reset_fru_error(int fru_location, tRUNTIME_FRU_ERROR *fruerror);

/*
 * add_entry_to_fru_error
 * ----------------------
 * Given the error code and error data, search the fru eror list
 * to see if this is an existing error.
 * If yes, update this entry with new error data and mark it not processed.
 * If no, create a new entry and apend it to the end of the list.
 */
void add_entry_to_fru_error(int fru_location,
                            tRUNTIME_ERR_DATA entrydata,
                            tRUNTIME_FRU_ERROR *fruerror);


/*
 * remove_entry_from_fru_error
 * ----------------------
 * Given the error code, search the fru eror list
 * to see if this is an existing error.
 * If found AND this node is marked processed, remove this node.
 * Otherwise, ignore.
 * This routine is intended to remove recovery event that has been processed.
 */
void remove_entry_from_fru_error(int fru_location,
                            unsigned short errcode_key,
                            tRUNTIME_FRU_ERROR *fruerror);

/*
 * process_entry_from_fru_error
 * ----------------------
 * Search the fru eror list for the next unprocessed entry.
 * Return 1 if there is an unprocessed entry, and return the data in this entry.
 * Return 0 otherwise.
 */
int process_entry_from_fru_error(int fru_location,
                            tRUNTIME_ERR_DATA *entrydata,
                            tRUNTIME_FRU_ERROR *fruerror);

/*
 * retrieve_all_processed_entry
 * ----------------------------
 * Go thru the list and return all entry data that have been processed.
 * This routine is intended for showing all fru errors.
 * Unprocessed entries will not be returned because thet need to be processed first
 * to be known to the system.
 * Entry data will be returned in the provided buffer. The size of the buffer is specified in max_num_entries.
 * The real numner of returned entries  will be returned in returned_num_entries.
 */
void retrieve_all_processed_entry(int fru_location,
                                  int max_num_entries,
                                  int *returned_num_entries,
                                  tRUNTIME_ERR_DATA *entrydata,
                                  tRUNTIME_FRU_ERROR *fruerror);



/*
 * make_fru_error
 * ---------------
 * Make a new fru error list based on the data ptovided.
 * Mark all entries processed.
 * This routine is intended to be used for data syncing bwtween 2 cores.
 */
void make_fru_error(int fru_location,
                    int num_entries,
                    tRUNTIME_ERR_DATA *entrydata,
                    tRUNTIME_FRU_ERROR *fruerror);






#endif _FRUERROR_H

