#include <linux/config.h>
#include <linux/version.h>
#include <linux/module.h>
#include <linux/poll.h>
#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/init.h>
#include <asm/uaccess.h>
#include <asm/io.h>
#include <asm/irq.h>
#include <asm/machdep.h>
#include <asm/ocp.h>
#include "gateway_sysevent.h"


extern int global_i2c_debug;

int sysevent_init_module(void);
void sysevent_cleanup_module(void);


/* Function Prototypes */
static ssize_t sysevent_read(struct file *file, char *buf, size_t count, loff_t *offset);
static ssize_t sysevent_write(struct file *file, const char *buf, size_t count, loff_t *offset);



/* File Operation Table */
static struct file_operations char_fops = {
    owner: THIS_MODULE,
    read: sysevent_read,
    write: sysevent_write,
};


int sysevent_init_module(void)
{

    /* register char device */
    if (register_chrdev(SYSEVENT_MAJOR, "syseventmodule", &char_fops)) {
        printk(KERN_ERR "Unable to register syseventmodule\n");
        return -EIO;
    }

    return 0;
}


void sysevent_cleanup_module(void)
{

    unregister_chrdev(SYSEVENT_MAJOR, "syseventmodule");

    return;
}


/* Read routine */
static ssize_t sysevent_read(struct file *file, char *buf, size_t count, loff_t *offset)
{
    int dummy = 0;

    copy_to_user(buf, &dummy, sizeof(int));
    return sizeof(int);

}


static ssize_t sysevent_write(struct file *file, const char *buf, size_t count, loff_t *offset)
{
    int value;

    copy_from_user(&value, buf, sizeof(int));

    if ((value & I2C_DEBUG_CODE) == I2C_DEBUG_CODE) {
        global_i2c_debug = value & 0xff;
    }

    return sizeof(int);

}

    


