/*
 *
 *    Copyrigh t(c) 1999-2000 Grant Erickson <grant@lcse.umn.edu>
 *
 *    Copyright 2000-2001 MontaVista Software Inc.
 *      Completed implementation.
 *      Author: MontaVista Software, Inc.  <source@mvista.com>
 *
 *    Module name: ts-igr-sc.c
 *
 *    Description:
 *      Architecture- / platform-specific boot-time initialization code for
 *      IBM PowerPC 4xx based boards. Adapted from original
 *      code by Gary Thomas, Cort Dougan <cort@fsmlabs.com>, and Dan Malek
 *      <dan@net4x.com>.
 *
 *      History: 11/09/2001 - armin
 *      added board_init to add in additional instuctions needed during platfrom_init
 *
 *      01/22/2002 - Armin
 *      converted pci to ocp
 *
 *
 */
#include <linux/config.h>
#include <linux/init.h>
#include <linux/smp.h>
#include <linux/threads.h>
#include <linux/param.h>
#include <linux/string.h>
#include <linux/blk.h>
#include <linux/pci.h>
#include <linux/rtc.h>

#include <asm/system.h>
#include <asm/pci-bridge.h>
#include <asm/processor.h>
#include <asm/machdep.h>
#include <asm/page.h>
#include <asm/time.h>
#include <asm/io.h>
#include <platforms/ibm_ocp.h>

#ifdef CONFIG_PPC_RTC
#include <asm/todc.h>
#endif

#undef DEBUG

#ifdef DEBUG
#define DBG(x...) printk(x)
#else
#define DBG(x...)
#endif

void *kb_cs;
void *kb_data;
void *walnut_rtc_base;

/* Global variables */
struct semaphore i2c_lock;
unsigned char i2c_controller_slave_addr;
int i2c_controller_fast_mode;


/* Some IRQs unique to Walnut.
 * Used by the generic 405 PCI setup functions in ppc4xx_pci.c
 */
int __init
ppc405_map_irq(struct pci_dev *dev, unsigned char idsel, unsigned char pin)
{
        enum {
                NONE = -1,
                IRQ0 = 25,
                IRQ1 = 26,
                IRQ2 = 27,
                IRQ3 = 28,
                IRQ4 = 29,
                IRQ5 = 30,
                IRQ6 = 31
        };

	static char pci_irq_table[][4] =
	    /*
	     *      PCI IDSEL/INTPIN->INTLINE
	     *      A       B       C       D
	     */
	{
		{NONE, NONE, NONE, NONE}, /* IDSEL 1 */
		{NONE, NONE, NONE, NONE}, /* IDSEL 2 */
		{NONE, NONE, NONE, NONE}, /* IDSEL 3 */
		{NONE, NONE, NONE, NONE}, /* IDSEL 4 */
		{NONE, NONE, NONE, NONE}, /* IDSEL 5 */
		{IRQ2, IRQ2, IRQ2, IRQ2}, /* IDSEL 6: Anafa 1 */
		{IRQ3, IRQ3, IRQ3, IRQ3}, /* IDSEL 7: Anafa 6 */
		{IRQ1, IRQ1, IRQ1, IRQ1}, /* IDSEL 8: Opti USB */
	};

	const long min_idsel = 1, max_idsel = 8, irqs_per_slot = 4;
	return PCI_IRQ_TABLE_LOOKUP;
};

void __init
board_setup_arch(void)
{
#ifdef CONFIG_PPC_RTC
	/* RTC step for the walnut */
	walnut_rtc_base = (void *) WALNUT_RTC_VADDR;
	TODC_INIT(TODC_TYPE_DS1743, walnut_rtc_base, walnut_rtc_base,
		  walnut_rtc_base, 8);
#endif				/* CONFIG_PPC_RTC */
}

void __init
bios_fixup(struct pci_controller *hose, void *pcil0_base)
{

	unsigned int bar_response, bar;
	struct pcil0_regs *pcip;
	/*
	 * Expected PCI mapping:
	 *
	 *  PLB addr             PCI memory addr
	 *  ---------------------       ---------------------
	 *  0000'0000 - 7fff'ffff <---  0000'0000 - 7fff'ffff
	 *  8000'0000 - Bfff'ffff --->  8000'0000 - Bfff'ffff
	 *
	 *  PLB addr             PCI io addr
	 *  ---------------------       ---------------------
	 *  e800'0000 - e800'ffff --->  0000'0000 - 0001'0000
	 *
	 * The following code is simplified by assuming that the bootrom
	 * has been well behaved in following this mapping.
	 */

#ifdef DEBUG
	int i;
	pcip = (struct pcil0_regs *) pcil0_base;

	printk("ioremap PCLIO_BASE = 0x%x\n", pcip);
	printk("PCI bridge regs before fixup \n");
	for (i = 0; i <= 3; i++) {
		printk(" pmm%dma\t0x%x\n", i, in_le32(&(pcip->pmm[i].ma)));
		printk(" pmm%dma\t0x%x\n", i, in_le32(&(pcip->pmm[i].la)));
		printk(" pmm%dma\t0x%x\n", i, in_le32(&(pcip->pmm[i].pcila)));
		printk(" pmm%dma\t0x%x\n", i, in_le32(&(pcip->pmm[i].pciha)));
	}
	printk(" ptm1ms\t0x%x\n", in_le32(&(pcip->ptm1ms)));
	printk(" ptm1la\t0x%x\n", in_le32(&(pcip->ptm1la)));
	printk(" ptm2ms\t0x%x\n", in_le32(&(pcip->ptm2ms)));
	printk(" ptm2la\t0x%x\n", in_le32(&(pcip->ptm2la)));

#else
	pcip = (struct pcil0_regs *) pcil0_base;
#endif

#ifdef __USE_IBM_OPENBIOS__ 
	/* added for IBM boot rom version 1.15 bios bar changes  -AK */

	/* Disable region first */
	out_le32((void *) &(pcip->pmm[0].ma), 0x00000000);
	/* PLB starting addr, PCI: 0x80000000 */
	out_le32((void *) &(pcip->pmm[0].la), 0x80000000);
	/* PCI start addr, 0x80000000 */
	out_le32((void *) &(pcip->pmm[0].pcila), PPC405_PCI_MEM_BASE);
	/* 512MB range of PLB to PCI */
	out_le32((void *) &(pcip->pmm[0].pciha), 0x00000000);
	/* Enable no pre-fetch, enable region */
	out_le32((void *) &(pcip->pmm[0].ma), ((0xffffffff -
						(PPC405_PCI_UPPER_MEM -
						 PPC405_PCI_MEM_BASE)) | 0x01));

	/* Disable region one */
	out_le32((void *) &(pcip->pmm[1].ma), 0x00000000);
	out_le32((void *) &(pcip->pmm[1].la), 0x00000000);
	out_le32((void *) &(pcip->pmm[1].pcila), 0x00000000);
	out_le32((void *) &(pcip->pmm[1].pciha), 0x00000000);
	out_le32((void *) &(pcip->pmm[1].ma), 0x00000000);
	out_le32((void *) &(pcip->ptm1ms), 0x00000000); 

	/* Disable region two */
	out_le32((void *) &(pcip->pmm[2].ma), 0x00000000);
	out_le32((void *) &(pcip->pmm[2].la), 0x00000000);
	out_le32((void *) &(pcip->pmm[2].pcila), 0x00000000);
	out_le32((void *) &(pcip->pmm[2].pciha), 0x00000000);
	out_le32((void *) &(pcip->pmm[2].ma), 0x00000000);
	out_le32((void *) &(pcip->ptm2ms), 0x00000000);

	/* Zero config bars */
	for (bar = PCI_BASE_ADDRESS_1; bar <= PCI_BASE_ADDRESS_2; bar += 4) {
		early_write_config_dword(hose, hose->first_busno,
					 PCI_FUNC(hose->first_busno), bar,
					 0x00000000);
		early_read_config_dword(hose, hose->first_busno,
					PCI_FUNC(hose->first_busno), bar,
					&bar_response);
		DBG("BUS %d, device %d, Function %d bar 0x%8.8x is 0x%8.8x\n",
		    hose->first_busno, PCI_SLOT(hose->first_busno),
		    PCI_FUNC(hose->first_busno), bar, bar_response);
	}
	/* end work arround */
#endif /* __USE_IBM_OPENBIOS__ */

#ifdef DEBUG
	printk("PCI bridge regs after fixup \n");
	for (i = 0; i <= 3; i++) {
		printk(" pmm%dma\t0x%x\n", i, in_le32(&(pcip->pmm[i].ma)));
		printk(" pmm%dma\t0x%x\n", i, in_le32(&(pcip->pmm[i].la)));
		printk(" pmm%dma\t0x%x\n", i, in_le32(&(pcip->pmm[i].pcila)));
		printk(" pmm%dma\t0x%x\n", i, in_le32(&(pcip->pmm[i].pciha)));
	}
	printk(" ptm1ms\t0x%x\n", in_le32(&(pcip->ptm1ms)));
	printk(" ptm1la\t0x%x\n", in_le32(&(pcip->ptm1la)));
	printk(" ptm2ms\t0x%x\n", in_le32(&(pcip->ptm2ms)));
	printk(" ptm2la\t0x%x\n", in_le32(&(pcip->ptm2la)));

#endif
}

void __init
board_io_mapping(void)
{
	io_block_mapping(WALNUT_RTC_VADDR,
			 WALNUT_RTC_PADDR, WALNUT_RTC_SIZE, _PAGE_IO);
}

void __init
board_setup_irq(void)
{
        unsigned long polarity;
        unsigned long trigger;

        /* Set PCI interrupts to be level triggered, active low */

        /* set polarity of external IRQs 0 thru 7 to negative */
        polarity = mfdcr(DCRN_UIC_PR(UIC0));
        polarity &= 0xffffff80UL;
        mtdcr(DCRN_UIC_PR(UIC0), polarity);
        printk(KERN_INFO "set UIC0_PR to 0x%08x (tried 0x%08x)\n",
               mfdcr(DCRN_UIC_PR(UIC0)), polarity);

        /* set trigger to level for IRQs 0 thru 7 */
        trigger = mfdcr(DCRN_UIC_TR(UIC0));
        trigger &= 0xffffff80UL;
        mtdcr(DCRN_UIC_TR(UIC0), trigger);
        printk(KERN_INFO "set UIC0_TR to 0x%08x (tried 0x%08x)\n",
               mfdcr(DCRN_UIC_TR(UIC0)), trigger);

	/* Clear any external interrupts caused by polarity miss match */
        mtdcr(DCRN_UIC_SR(UIC0), 0x0000007fUL);
}

void __init
board_init(void)
{
        /* Use external clock for serial ports */
  	mtdcr(DCRN_CHCR0,
              mfdcr(DCRN_CHCR0) | CHR0_U0EC | CHR0_U1EC);

    init_MUTEX(&i2c_lock);

    i2c_controller_slave_addr = 0;            /* not in slave mode */
    i2c_controller_fast_mode = 0;             /* slow mode 100 KHz */


#ifdef CONFIG_PPC_RTC
	ppc_md.time_init = todc_time_init;
	ppc_md.set_rtc_time = todc_set_rtc_time;
	ppc_md.get_rtc_time = todc_get_rtc_time;
	ppc_md.nvram_read_val = todc_direct_read_val;
	ppc_md.nvram_write_val = todc_direct_write_val;
#endif
}
