/*
 *  __dma_buffer macro for safe DMA into struct members
 *
 *  See "What memory is DMA'able?" in Documentation/DMA-mapping.txt
 *  for more information.
 *
 *  Copyright (c) 2002  Roland Dreier <roland@digitalvampire.org>
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version
 *  2 of the License, or (at your option) any later version.  */

#ifdef __KERNEL__
#ifndef _ASM_PPC_DMA_BUFFER_H
#define _ASM_PPC_DMA_BUFFER_H

#include <linux/config.h>

#ifdef CONFIG_NOT_COHERENT_CACHE

#include <asm/cache.h>

#define __dma_buffer __dma_buffer_line(__LINE__)
#define __dma_buffer_line(line) __dma_buffer_expand_line(line)
#define __dma_buffer_expand_line(line) \
	__attribute__ ((aligned(L1_CACHE_BYTES))); \
	char __dma_pad_ ## line [0] __attribute__ ((aligned(L1_CACHE_BYTES)))

#else  /* CONFIG_NOT_COHERENT_CACHE */

/* We have cache-coherent DMA so __dma_buffer is defined to be a NOP */
#define __dma_buffer

#endif /* CONFIG_NOT_COHERENT_CACHE */

#endif /* _ASM_PPC_DMA_BUFFER_H */
#endif /* __KERNEL__ */
