/***************************************************************************
*
*   Module:				PPCBoot/board/ts_igr_lt
*
*   Filename:           fan.c
*
*   Author:             Brian Tran
*
*   Created:            01/08/2003
*
*   Description:
*
*   For fan controller ADM1031.
*
****************************************************************************
*                 Source Control System Information
*
*    $Id: fan.c,v 1.8 2003/07/08 20:21:15 roland Exp $
*
****************************************************************************
*                       Modification History
*
*    $Log: fan.c,v $
*    Revision 1.8  2003/07/08 20:21:15  roland
*    Get rid of references to IGR
*
*    Revision 1.7  2003/02/27 00:24:16  sarah
*    don't set tach limit for phantom fan on cntrl 0
*
*    Revision 1.6  2003/02/07 02:20:02  sarah
*    don't do post, only if dopost = 0
*
*    Revision 1.5  2003/02/05 22:33:12  sarah
*    set tach limit for auto mode
*
*    Revision 1.4  2003/01/24 23:03:39  sarah
*    add auto fan control
*
*    Revision 1.3  2003/01/20 18:17:22  sarah
*    fix fan_present
*
*    Revision 1.2  2003/01/15 01:44:58  sarah
*    additions for POST
*
*    Revision 1.1  2003/01/08 18:29:34  sarah
*    code to turn fans on
*
*
***************************************************************************
*                       (c) COPYRIGHT 2000, 2001, 2002
*                           TopSpin Communications, Inc.
*                           Palo Alto, CA
**************************************************************************/
#ifdef RCSID
static const char rcsid[] = "$Id: fan.c,v 1.8 2003/07/08 20:21:15 roland Exp $";
#endif

#include <ppcboot.h>
#include "../../include/i2c.h"
#include "fpga.h"
#include "fan.h"

const uchar fan_addr[] = {0x2c, 0x2d};

extern int dontboot;

int POST_fan = 0;
 
/* fan_write
 * --------
 */
int fan_write(int fanctlr, uint offset, uchar data)
{
    uchar device_addr;
    int return_code;

    device_addr = fan_addr[fanctlr];

    return_code = i2c_write(device_addr, offset, 1, &data, 1);

    if (return_code != 0) {
        printf ("fan ctlr%d: i2c_write 0x%x %d %d 0x%x %d FAILED\n", fanctlr,
            fan_addr[fanctlr], offset, 1, (int)&data, 1);
        return 1;
    }

    return 0;
}

/* fan_read
 * --------
 */
int fan_read(int fanctlr, uchar offset, uchar *read)
{
    int device_addr;
    int return_code;
    uchar buf;

    device_addr = fan_addr[fanctlr];

    return_code = i2c_read(device_addr, offset, 1, &buf, 1);

//    printf ("fan_read: 0x%x=0x%x \n", device_addr, buf);
    *read = buf;
    return (return_code);

}

/* fan_present
 * -----------
 */
int fan_present(void)
{
    uchar reg1;

    reg1 = in8(LT_FPGA_BASE + LT_FPGA_REG1_OFFSET);

    /* fan present 0 = present */
    if ((reg1 & LT_FPGA_FAN_PRESENT_MASK) == 0)
        return 1;  /* present */
    else
        return 0;  /* not present */
}


/* fanconfig
 * --------
 * Soft reset (Config Init) the LM, just in case it was screwed up before.
 * Set Watchdog limits. For voltages, limits are +-8% of nominal 3.3V, 2.5V, and 1.8V. Temperature limits are 65oC and -10oC.
 * Set interrupt: enable interrupt (Config), active low open drain, enable mask1 and 2.
 * Start the monitoring process.
 * Note: all voltages pass thru a divider so that LM reads 1.8 V at all inputs. 
 * Therefore, +-8% tolerance will be seen differently at LM inputs (scaled by the divider).  
 * The 8-bit ADC is 10mV LSB.
 * Use default temp resolution is 9-bit.
 */ 
int fanconfig(int fanctlr, int dopost)
{
   uchar fan_config;

    /* soft reset. Write 1 to bit 7 of config 2 reg. This bit will clear itself after the LM is reset */
    if (fan_write(fanctlr, ADM_CONFIG2_OFFSET, ADM_CONFIG2_DEFAULT | 0x80) != 0) {
        return 1;
    }

    udelay(1000);

    /* default config */
    if (dopost) {
        printf ("Setting fan cntlr%d to MANUAL mode\n", fanctlr);
        fan_config = ADM_CONFIG1_DEFAULT;
    } else { 
        printf ("Setting fan cntlr%d to AUTO mode\n", fanctlr);
        fan_config = ADM_CONFIG1_DEFAULT_AUTO;
        if (fan_write(fanctlr, ADM_LOCALTEMP_CONFIG_OFFSET, ADM_LOCALTEMP_CONFIG_DEFAULT) != 0)
            return 1;

        /* set TAC LIMIT */
        if (fan_write(fanctlr, ADM_FAN1_TACHHI_LIMIT_OFFSET, ADM_TACHHI_LIMIT) != 0) { 
            return 1;
        }
        /* fanctlr has 2 fans */
        if (fanctlr == 1) {
            if (fan_write(fanctlr, ADM_FAN2_TACHHI_LIMIT_OFFSET, ADM_TACHHI_LIMIT) != 0) {
                return 1;
            }
        }
    }
    if (fan_write(fanctlr, ADM_CONFIG1_OFFSET, fan_config) != 0)
        return 1;

    if (fan_write(fanctlr, ADM_CONFIG2_OFFSET, ADM_CONFIG2_DEFAULT) != 0) {
        return 1;
    }


    /* fan characteristics */
    if (fan_write(fanctlr, ADM_FAN_CHAR1_OFFSET, ADM_FAN_CHAR_DEFAULT) != 0) {
        return 1;
    }

    if (fan_write(fanctlr, ADM_FAN_CHAR2_OFFSET, ADM_FAN_CHAR_DEFAULT) != 0) {
        return 1;
    }

    /* thermal behavior */
    if (fan_write(fanctlr, ADM_THERM_BEHAVE_OFFSET, ADM_THERM_BEHAVE_DEFAULT) != 0) {
        return 1;
    }

    if (fan_write(fanctlr, ADM_LOCAL_THERM_LIMIT_OFFSET, 0) != 0) {
        return 1;
    }

    /* If POST, set to high speed, otherwise its set to auto, leave it */
    if (dopost) {
        if (fan_write(fanctlr, ADM_FAN_SPEED_CONFIG_OFFSET, ADM_FAN_SPEED_MAX_SETTING) != 0) {
            return 1;
        }
    } 

    /* start monitoring the temperature and fan speed */
    if (fan_write(fanctlr, ADM_CONFIG1_OFFSET, fan_config | 0x1) != 0) {
        return 1;
    }

    return 0;
}

/*
 * fanspeed
 * --------
 */
int fanspeed(int fanctlr, int percent_speed)
{
    uchar data;
    int i;
    int loop = 1;

    /* calculate data: 0 for 0%, f for 100 % */
    data = 0xf * percent_speed / 100;

    /* one nibble for one fan. Want to set two fans at the same speed */
    data |= data << 4;

    i = 0;
    do {

        if (fan_write(fanctlr, ADM_FAN_SPEED_CONFIG_OFFSET, data) != 0) {
            return 1;
        }

        i ++;
        /* check for infinite loop */
        if (loop == -1) {
            i = -2;
        }
    } while (i < loop);

    return 0;
}

/*
 * fangetspeed
 * -----------
 * Read fan count and convert to RPM.
 */
int fangetspeed(int fanctlr, int *fan1_rpm, int *fan2_rpm)
{
    uchar fan_count;
    int count = 0;
    int retry;

    for (retry = 0; retry < 20; retry ++) {
        if (fan_read(fanctlr, ADM_FAN1_SPEED_OFFSET, &fan_count) != 0) {
            printf ("POST: fan ctlr%d: error reading speed, count=0x%x: FAILED\n", fanctlr, count);
            return 1;
        }

        count = fan_count & 0xff;

        if (count != 0) {
            break;
        }

        udelay(100000);

    }

    if (count == 0) {
        printf ("POST: fan ctlr%d: error reading fan speed: FAILED\n", fanctlr);
        return 1;
    }

    *fan1_rpm = (ADM_FAN_SPEED_MEASURE_FREQ * 60) / (count * ADM_FAN_SPEED_RANGE);

    if (fanctlr == 0) /* IGRLite ctlr 0 has only 1 fan */
        return 0;

    if (fan_read(fanctlr, ADM_FAN2_SPEED_OFFSET, &fan_count) != 0) {
        printf ("POST: fan ctlr%d: error reading fan speed: FAILED\n", fanctlr);
        return 1;
    }
    count = fan_count & 0xff;
    *fan2_rpm = (ADM_FAN_SPEED_MEASURE_FREQ * 60) / (count * ADM_FAN_SPEED_RANGE);

    return 0;
}

/*
 * fanpost
 * -------
 */
int fanpost(int fanctlr)
{
    int fan1_rpm, fan2_rpm;
//	uchar fan_count;
    int fail = 0;
 
    printf("POST: ctlr%d wait for fans to speed-up...\n", fanctlr);
    udelay(10000000); /* fan_read works with this delay */
//    fan_read(fanctlr, ADM_FAN1_SPEED_OFFSET, &fan_count);

    /* set to max speed in fanconfig, check the speed */

    if (fangetspeed(fanctlr, &fan1_rpm, &fan2_rpm) != 0)
        return 1;

    if ((fan1_rpm > (ADM_FAN_SPEED_MAX + ADM_FAN_SPEED_DELTA)) || (fan1_rpm < (ADM_FAN_SPEED_MAX - ADM_FAN_SPEED_DELTA))) 
    {
        fail = 1;
        printf ("POST: fan ctlr%d: fan1 speed %d not high: FAILED\n", fanctlr, fan1_rpm);
    }

    if (fanctlr == 1) /* IGRLite fan ctlr 1 has 2 fans */
    {

        if ((fan2_rpm > (ADM_FAN_SPEED_MAX + ADM_FAN_SPEED_DELTA)) || (fan2_rpm < (ADM_FAN_SPEED_MAX - ADM_FAN_SPEED_DELTA))) 
        {
            fail = 1;
            printf ("POST: fan ctlr%d: fan2 speed %d not high: FAILED\n", fanctlr, fan2_rpm);
        }
    }

    printf("POST: ctlr%d wait for fans to slow-down...\n", fanctlr);

    /* set to min speed, then test */
    if (fanspeed(fanctlr, 20) != 0)
    {
        fail = 1;
        printf ("POST: fan cntlr%d speed set to 20 percent FAILED\n", fanctlr);
    }

    udelay(10000000); /* wait for fan to slow down */
    if (fangetspeed(fanctlr, &fan1_rpm, &fan2_rpm) != 0)
        return 1;

    if ((fan1_rpm < (ADM_FAN_SPEED_MIN - ADM_FAN_SPEED_DELTA)) || (fan1_rpm > (ADM_FAN_SPEED_MIN + ADM_FAN_SPEED_DELTA))) 
    {
        fail = 1;
        printf ("POST: fan ctlr%d: fan1 speed %d not low: FAILED\n", fanctlr, fan1_rpm);
    } 

    if (fanctlr == 1) /* IGRLite fan ctlr 1 has 2 fans */
    {
        if ((fan2_rpm < (ADM_FAN_SPEED_MIN - ADM_FAN_SPEED_DELTA)) || (fan2_rpm > (ADM_FAN_SPEED_MIN + ADM_FAN_SPEED_DELTA))) 
        {
            fail = 1;
            printf ("POST: fan ctlr%d: fan2 speed %d not low: FAILED\n", fanctlr, fan2_rpm);
        } 
    }


    return fail;
}

/* faninit
 * --------
 */
int faninit(void)
{
    char *s;
    int dopost = 1;

    printf ("TS90 cntlr0: 1 fan, cntlr1: 2 fans\n");

    s = getenv ("dopost");
    if (strcmp(s, "0") == 0)
        dopost = 0;

    if (dopost) {
        if (fan_present() == 0) {
    		dontboot = 1;
            printf ("POST: fans not present: FAILED\n");
            return 1;
        }
    }
    
    if (fanconfig(0, dopost) == 0) {
        if (dopost) {
            if (fanpost(0) != 0) {
				dontboot = 1;
                printf ("POST: fan cntlr%d FAILED\n", 0);
            } else {
                printf ("POST: fan cntlr%d PASSED\n", 0);
            }
        }
    } else {
		dontboot = 1;
        printf ("POST: fan cntlr%d init FAILED\n", 0);
    }

    if (fanconfig(1, dopost) == 0) {
        if (dopost) {
            if (fanpost(1) != 0) {
				dontboot = 1;
                printf ("POST: fan cntlr%d FAILED\n", 1);
            } else {
                printf ("POST: fan cntlr%d PASSED\n", 1);
            }
        }
    } else {
		dontboot = 1;
        printf ("POST: fan cntlr%d init FAILED\n", 1);
    }

    /* Reset fans to auto control */
    if (dopost) {
        fanconfig(0, 0);
        fanconfig(1, 0);
    }

    return 0;
}
