/* m4/a440core.inc, openbios_ebony, ebony_bios 9/14/99 09:15:36
!------------------------------------------------------------------------------+
!
!       This source code has been made available to you by IBM on an AS-IS
!       basis.  Anyone receiving this source is licensed under IBM
!       copyrights to use it in any way he or she deems fit, including
!       copying it, modifying it, compiling it, and redistributing it either
!       with or without modifications.  No license under IBM patents or
!       patent applications is to be implied by the copyright license.
!
!       Any user of this software should understand that IBM cannot provide
!       technical support for this software and will not be responsible for
!       any consequences resulting from the use of this software.
!
!       Any person who transfers this source code or any derivative work
!       must include the IBM copyright notice, this paragraph, and the
!       preceding two paragraphs in the transferred software.
!
!       COPYRIGHT   I B M   CORPORATION 1995
!       LICENSED MATERIAL  -  PROGRAM PROPERTY OF I B M
!-------------------------------------------------------------------------------
!-------------------------------------------------------------------------------
!
!  File Name:   a440core.inc
!
!  Function:    Defines for 440 core registers used in assembly language.
!
!  Author:      Neil Leeder
!
!  Change Activity-
!
!  Date        Description of Change                                       BY
!  ---------   ---------------------                                       ---
!  12-Apr-99   Created							   NL
!  05-Sep-00   Port to 440GP                                               DWG
!
!-------------------------------------------------------------------------------
*/
/*
!-------------------------------------------------------------------------------
! Special Purpose Registers
!-------------------------------------------------------------------------------
*/
#ifndef __A440CORE_INC__
#define __A440CORE_INC__

#ifdef __cplusplus
extern "C" {
#endif

#if 0
#define  XER     0x001      /* fixed point exception register */
#define  LR      0x008      /* link register */
#define  CTR     0x009      /* count register */
#define  DEC     0x016      /* decrementer timer */
#define  SRR0    0x01a      /* save/restore register 0 */
#define  SRR1    0x01b      /* save/restore register 1 */
#define  PID     0x030      /* process ID */
#define  DECAR   0x036      /* decrementer auto-reload */
#define  CSRR0   0x03a      /* critical save/restore register 0 */
#define  CSRR1   0x03b      /* critical save/restore register 1 */
#define  DEAR    0x03d      /* data exeption address register */
#define  ESR     0x03e      /* execption syndrome register */
#define  IVPR    0x03f      /* interrupt vector prefix register */
#define  USPRG0  0x100      /* user special purpose register general 0 */
#define  SPRG4R  0x104      /* Special purpose general 4 - read only */
#define  SPRG5R  0x105      /* Special purpose general 5 - read only */
#define  SPRG6R  0x106      /* Special purpose general 6 - read only */
#define  SPRG7R  0x107      /* Special purpose general 7 - read only */
#define  TBLR    0x10c      /* time base lower - user read */
#define  TBUR    0x10d      /* time base upper - user read */
#define  SPRG0   0x110      /* Special purpose general 0 */
#define  SPRG1   0x111      /* Special purpose general 1 */
#define  SPRG2   0x112      /* Special purpose general 2 */
#define  SPRG3   0x113      /* Special purpose general 3 */
#define  SPRG4W  0x114      /* Special purpose general 4 - write only */
#define  SPRG5W  0x115      /* Special purpose general 5 - write only */
#define  SPRG6W  0x116      /* Special purpose general 6 - write only */
#define  SPRG7W  0x117      /* Special purpose general 7 - write only */
#define  TBLW    0x11c      /* time base lower - privileged write */
#define  TBUW    0x11d      /* time base upper - privileged write */
#define  PIR     0x11e      /* processor identification register */
#define  PVR     0x11f      /* processor version register */
#define  DBSR    0x130      /* debug status register */
#define  DBCR0   0x134      /* debug control register 0 */
#define  DBCR1   0x135      /* debug control register 1 */
#define  DBCR2   0x136      /* debug control register 2 */
#define  IAC1    0x138      /* instruction address comparator 1 */
#define  IAC2    0x139      /* instruction address comparator 2 */
#define  IAC3    0x13a      /* instruction address comparator 3 */
#define  IAC4    0x13b      /* instruction address comparator 4 */
#define  DAC1    0x13c      /* data address comparator 1 */
#define  DAC2    0x13d      /* data address comparator 2 */
#define  DVC1    0x13e      /* data value compare register 1 */
#define  DVC2    0x13f      /* data value compare register 2 */
#define  TSR     0x150      /* timer status register */
#define  TCR     0x154      /* timer control register */
#define  IVOR0   0x190      /* interrupt vector offset reg - critical input */
#define  IVOR1   0x191      /* interrupt vector offset reg - machine check */
#define  IVOR2   0x192      /* interrupt vector offset reg - data storage */
#define  IVOR3   0x193      /* interrupt vector offset reg - instr storage */
#define  IVOR4   0x194      /* interrupt vector offset reg - ext input */
#define  IVOR5   0x195      /* interrupt vector offset reg - alignment */
#define  IVOR6   0x196      /* interrupt vector offset reg - program */
#define  IVOR7   0x197      /* interrupt vector offset reg - floating pt unavail */
#define  IVOR8   0x198      /* interrupt vector offset reg - system call */
#define  IVOR9   0x199      /* interrupt vector offset reg - aux proc unavail */
#define  IVOR10  0x19a      /* interrupt vector offset reg - decrementer */
#define  IVOR11  0x19b      /* interrupt vector offset reg - fixed int timer */
#define  IVOR12  0x19c      /* interrupt vector offset reg - watchdog timer */
#define  IVOR13  0x19d      /* interrupt vector offset reg - data tlb error */
#define  IVOR14  0x19e      /* interrupt vector offset reg - instr tlb error */
#define  IVOR15  0x19f      /* interrupt vector offset reg - debug */
#define  INV0    0x370      /* instruction cache normal victim 0 */
#define  INV1    0x371      /* instruction cache normal victim 1 */
#define  INV2    0x372      /* instruction cache normal victim 2 */
#define  INV3    0x373      /* instruction cache normal victim 3 */
#define  ITV0    0x374      /* instruction cache transient victim 0 */
#define  ITV1    0x375      /* instruction cache transient victim 1 */
#define  ITV2    0x376      /* instruction cache transient victim 2 */
#define  ITV3    0x377      /* instruction cache transient victim 3 */
#define  DNV0    0x390      /* data cache normal victim 0 */ 
#define  DNV1    0x391      /* data cache normal victim 1 */
#define  DNV2    0x392      /* data cache normal victim 2 */
#define  DNV3    0x393      /* data cache normal victim 3 */
#define  DTV0    0x394      /* data cache transient victim 0 */
#define  DTV1    0x395      /* data cache transient victim 1 */
#define  DTV2    0x396      /* data cache transient victim 2 */
#define  DTV3    0x397      /* data cache transient victim 3 */
#define  DVLIM   0x398      /* data cache victim limit */
#define  IVLIM   0x399      /* instruction cache victim limit */
#define  RSTCFG  0x39b      /* reset configuration */
#define  DCDBTRL 0x39c      /* data cache debug tag register low */
#define  DCDBTRH 0x39d      /* data cache debug tag register high */ 
#define  ICDBTRL 0x39e      /* instruction cache debug tag register low */
#define  ICDBTRH 0x39f      /* instruction cache debug tag register high */
#define  MMUCR   0x3b2      /* memory management control register */
#define  CCR0    0x3b3      /* core configuration register */
#define  ICDBDR  0x3d3      /* instruction cache data debug register */
#define  DBDR    0x3f3      /* debug data register */
 
/* Dummy defines for registers which have no SPR number */
#define  IAR     2000       /* instruction address register */
#define  MSR     2001       /* machine state register       */
#define  CR      2002       /* condition register           */


/*-----------------------------------------------------------------------------+
| Macro defines for general purpose registers, all 32-bit, sequential no's.
+-----------------------------------------------------------------------------*/
#define  GPR0    0
#define  GPR1    1
#define  GPR2    2
#define  GPR3    3
#define  GPR4    4
#define  GPR5    5
#define  GPR6    6
#define  GPR7    7
#define  GPR8    8
#define  GPR9    9
#define  GPR10   10
#define  GPR11   11
#define  GPR12   12
#define  GPR13   13
#define  GPR14   14
#define  GPR15   15
#define  GPR16   16
#define  GPR17   17
#define  GPR18   18
#define  GPR19   19
#define  GPR20   20
#define  GPR21   21
#define  GPR22   22
#define  GPR23   23
#define  GPR24   24
#define  GPR25   25
#define  GPR26   26
#define  GPR27   27
#define  GPR28   28
#define  GPR29   29
#define  GPR30   30
#define  GPR31   31
#endif

/*-----------------------------------------------------------------------------+
| 440GP PVR.
+-----------------------------------------------------------------------------*/
#define  PVR_440GP            0x40120000
#define  PVR_440GP_PASS_1_0   0x40120040  /* RevA */
#define  PVR_440GP_PASS_1_1   0x40120440  /* RevA */
#define  PVR_440GP_PASS_2_0   0x40120481  /* RevB */

#if 0
/*-----------------------------------------------------------------------------+
| Machine State Register     msr     Bit Masks
+-----------------------------------------------------------------------------*/
#define  MSR_POW 0x00040000  /* Activates power management      */
#define  MSR_WE  0x00040000  /* wait state enable */
#define  MSR_CE  0x00020000  /* critical interrupt enable */
#define  MSR_ILE 0x00010000  /* Interrupt little Endian         */
#define  MSR_EE  0x00008000  /* external interrupt              */
#define  MSR_PR  0x00004000  /* problem state                   */
#define  MSR_FP  0x00002000  /* floating point available        */
#define  MSR_ME  0x00001000  /* machine check                   */
#define  MSR_FE0 0x00000800  /* floating point exception enable */
#define  MSR_SE  0x00000400  /* single step trace enable        */
#define  MSR_BE  0x00000200  /* branch trace enable             */
#define  MSR_DE  0x00000200  /* debug enable */
#define  MSR_FE1 0x00000100  /* floating point exception enable */
#define  MSR_EP  0x00000040  /* prefix                          */
#define  MSR_IP  0x00000040  /* prefix                          */
#define  MSR_IR  0x00000020  /* instruction relocate            */
#define  MSR_DR  0x00000010  /* data relocate                   */
#define  MSR_RI  0x00000002  /* Recoverable interrupt           */
#define  MSR_LE  0x00000001  /* Little Endian                   */
#endif

/*-----------------------------------------------------------------------------+
| MSR bit masks
+-----------------------------------------------------------------------------*/
#define ppcMsrWE        0x00040000      /* Wait State Enable               */
#define ppcMsrCE        0x00020000      /* Critical Interrupt Enable       */
#define ppcMsrEE        0x00008000      /* External Interrupt Enable       */
#define ppcMsrPR        0x00004000      /* Problem State                   */
#define ppcMsrFP        0x00002000      /* Floating-Point Available        */
#define ppcMsrME        0x00001000      /* Machine Check Enable            */
#define ppcMsrFE0       0x00000800      /* Floating-Point Mode 0           */
#define ppcMsrDWE	0x00000400	/* Debug wait enable               */
#define ppcMsrDE        0x00000200      /* Debug Exeption Enable           */
#define ppcMsrFE1       0x00000100      /* Floating-Point Mode 1           */
#define ppcMsrIS        0x00000020      /* Instruction Address Space       */
#define ppcMsrDS        0x00000010      /* Data Address Space              */

/*-----------------------------------------------------------------------------+
| Timer status register TSR.
+-----------------------------------------------------------------------------*/
#define TSR_ENW      0x80000000         /* next Watchdog event goverened by 
                                           TSR(1)  */
#define TSR_WIS      0x40000000		/* Watchdog interrupt status          */
#define TSR_WRS_MASK 0x30000000		/* Watchdog reset mask                */
#define TSR_WRS_CORE 0x10000000		/* Core reset forced by watchdog      */
#define TSR_WRS_CHIP 0x20000000		/* Chip reset forced by watchdog      */
#define TSR_WRS_SYS  0x30000000		/* System reset forced by watchdog    */
#define TSR_DIS	     0x08000000		/* DEC interrupt pending              */
#define TSR_FIS	     0x04000000		/* FIS interrupt pending 	      */

/*-----------------------------------------------------------------------------+
| Timer control register.
+-----------------------------------------------------------------------------*/
#define TCR_WP_MASK     0xC0000000	/* Watchdog period mask               */
#define TCR_WP_2_21     0x00000000	/* Period is 2**21 clocks             */
#define TCR_WP_2_25     0x40000000	/* Period is 2**25 clocks             */
#define TCR_WP_2_29     0x80000000	/* Period is 2**29 clocks             */
#define TCR_WP_2_33     0xC0000000	/* Period is 2**33 clocks             */
#define TCR_WRC_MASK    0x30000000	/* Watchdog reset mask                */
#define TCR_WRC_CORE    0x10000000	/* Core reset forced by watchdog      */
#define TCR_WRC_CHIP    0x20000000	/* Chip reset forced by watchdog      */
#define TCR_WRC_SYS     0x30000000	/* System reset forced by watchdog    */
#define TCR_WIE         0x08000000	/* Watchdog interrupt enable          */
#define TCR_DIE        	0x04000000	/* DEC interrupt enable               */
#define TCR_FP_MASK   	0x03000000	/* Fixed interval period mask         */
#define TCR_FP_2_13     0x00000000	/* Period is 2**13 clocks             */
#define TCR_FP_2_17     0x01000000	/* Period is 2**17 clocks             */
#define TCR_FP_2_21     0x02000000	/* Period is 2**21 clocks             */
#define TCR_FP_2_25     0x03000000	/* Period is 2**25 clocks             */
#define TCR_FIE         0x00800000	/* FIT interrupt enable               */
#define TCR_ARE        	0x00400000	/* Auto-reload enable                 */

/*-----------------------------------------------------------------------------+
| Debug status register.
+-----------------------------------------------------------------------------*/
#define DBSR_IDE	0x80000000	/* Imprecise debug event              */
#define DBSR_UDE	0x40000000	/* Unconditional debug event          */
#define DBSR_MRR_MASK   0x30000000	/* Most recent reset                  */
#define DBSR_MRR_CORE 	0x10000000	/* Core reset                         */
#define DBSR_MRR_CHIP 	0x20000000	/* Chip reset                         */
#define DBSR_MRR_SYS  	0x30000000	/* System reset                       */
#define DBSR_ICMP	0x08000000	/* Instruction completion dbg event   */
#define DBSR_BRT	0x04000000	/* Branch taken debug event           */
#define DBSR_IRPT	0x02000000	/* Interrupt debug event              */
#define DBSR_TRAP	0x01000000	/* Trap debug event                   */
#define DBSR_IAC1	0x00800000	/* IAC 1 debug event          	      */
#define DBSR_IAC2	0x00400000	/* IAC 2 debug event                  */
#define DBSR_IAC3	0x00200000	/* IAC 3 debug event          	      */
#define DBSR_IAC4	0x00100000	/* IAC 4 debug event                  */
#define DBSR_DAC1R	0x00080000	/* DAC 1 read debug event             */
#define DBSR_DAC1W	0x00040000	/* DAC 1 write debug event            */
#define DBSR_DAC2R	0x00020000	/* DAC 2 read debug event             */
#define DBSR_DAC2W	0x00010000	/* DAC 2 write debug event            */
#define DBSR_RET	0x00008000	/* Return debug event                 */
#define DBSR_IAC12ATS	0x00000002	/* IAC 1/2 auto toggle status         */
#define DBSR_IAC34ATS	0x00000001	/* IAC 3/4 auto toggle status         */

/*-----------------------------------------------------------------------------+
|  SAVE/RESTORE REGISTERS 1 and 3 
+-----------------------------------------------------------------------------*/
#define SRR_WE          0x00040000      /* Wait enable                        */
#define SRR_CE          0x00020000      /* Critical interrupt enable          */
#define SRR_EE          0x00008000      /* External interrupt enable          */
#define SRR_PR          0x00004000      /* Problem state                      */
#define SRR_FP          0x00002000
#define SRR_ME          0x00001000      /* Machine check enable               */
#define SRR_FE0         0x00000800
#define SRR_DWE         0x00000400      /* Debug wait enable                  */
#define SRR_DE          0x00000200      /* Debug exception enable             */
#define SRR_FE1         0x00000100
#define SRR_IS          0x00000020      /* Instruction side                   */
#define SRR_DS          0x00000010      /* Data side                          */

/*-----------------------------------------------------------------------------+
|  Instruction Cache Debug Data Register for 440
+-----------------------------------------------------------------------------*/
#define ICDBDR_LK       0x00000400      /* Cache line lock                    */
#define ICDBDR_VAL      0x00000100      /* Instruction Cache Valid            */
#define ICDBDR_LRU      0x00000001      /* Instruction Cache LRU              */

/*-----------------------------------------------------------------------------+
| Core Configuration Register 0 for 440
+-----------------------------------------------------------------------------*/
#define CCR0_DSTG	0x00200000	/* Disable store gathering            */
#define CCR0_DAPUIB     0x00100000	/* Disable APU instruction broadcast  */
#define CCR0_DTB 	0x00008000	/* Disable trace broadcast            */
#define CCR0_GICBT      0x00004000	/* Guaranteed instr cache block touch */
#define CCR0_GDCBT      0x00002000	/* Guaranteed data cache block touch  */
#define CCR0_FLSTA      0x00000100	/* Force load/store alignment         */
#define CCR0_BTAC_DISABLE 0x00000040
#define CCR0_ICSLC_MASK 0x0000000C	/* IC speculative line count          */
#define CCR0_ICSLT_MASK 0x00000003	/* IC speculative line threshold      */

/*-----------------------------------------------------------------------------+
| Debug control register 0 for 440
+-----------------------------------------------------------------------------*/
#define DBCR0_EDM	0x80000000	/* External debug mode                */
#define DBCR0_IDM	0x40000000	/* Internal debug mode                */
#define DBCR0_RST_MASK  0x30000000
#define DBCR0_RST_CORE  0x10000000	/* Core reset                         */
#define DBCR0_RST_CHIP  0x20000000	/* Chip reset                         */
#define DBCR0_RST_SYS   0x30000000	/* system reset                       */
#define DBCR0_ICMP      0x08000000	/* instruction complete event         */
#define DBCR0_BRT       0x04000000	/* Branch taken debug event           */
#define DBCR0_IRPT      0x02000000	/* Interrupt debug event              */
#define DBCR0_TRAP      0x01000000	/* Trap Debug event                   */
#define DBCR0_IAC1	0x00800000	/* Instruction compare 1              */
#define DBCR0_IAC2	0x00400000	/* Instruction compare 2              */
#define DBCR0_IAC3	0x00200000	/* Instruction compare 3              */
#define DBCR0_IAC4 	0x00100000	/* Instruction compare 4              */
#define DBCR0_DAC1R     0x00080000	/* Data addr compare read debug event */
#define DBCR0_DAC1W     0x00040000	/* Data addr compare write debug event*/
#define DBCR0_DAC2R     0x00020000	/* Data addr compare read debug event */
#define DBCR0_DAC2W	0x00010000	/* Data addr cmopare write debug event*/
#define DBCR0_FT	0x00000001	/* Freeze timers                      */

/*-----------------------------------------------------------------------------+
| MMU control register 1 for 440
+-----------------------------------------------------------------------------*/
#define MMUCR_STID        0x000000FF
#define MMUCR_STID_ENCODE(x) (((x)&0xFF)<<0)
#define MMUCR_STID_DECODE(x) (((x)>>0)&0xFF)
#define MAX_TLB_ENTRIES 64              /* max number of TLB entries          */

#define MMU_WS0_EPN_MASK  0xFFFFFC00
#define MMU_WS0_EPN_ENCODE(x) (((x)&0x003FFFFF)<<10)
#define MMU_WS0_EPN_DECODE(x) (((x)>>10)&0x003FFFFF)
#define MMU_WS0_V         0x00000200
#define MMU_WS0_TS        0x00000100
#define MMU_WS0_SIZE_MASK 0x000000F0
#define MMU_WS0_SIZE_ENCODE(x) (((x)&0xF)<<4)
#define MMU_WS0_SIZE_DECODE(x) (((x)>>4)&0xF)

#define MMU_WS1_RPN_MASK  0xFFFFFC00
#define MMU_WS1_RPN_ENCODE(x) (((x)&0x003FFFFF)<<10)
#define MMU_WS1_RPN_DECODE(x) (((x)>>10)&0x003FFFFF)
#define MMU_WS1_ERPN_MASK 0x0000000F
#define MMU_WS1_ERPN_ENCODE(x) (((x)&0xF)<<0)
#define MMU_WS1_ERPN_DECODE(x) (((x)>>0)&0xF)

#define MMU_WS2_U0        0x00008000
#define MMU_WS2_U1        0x00004000
#define MMU_WS2_U2        0x00002000
#define MMU_WS2_U3        0x00001000
#define MMU_WS2_W         0x00000800
#define MMU_WS2_I         0x00000400
#define MMU_WS2_M         0x00000200
#define MMU_WS2_G         0x00000100
#define MMU_WS2_E         0x00000080
#define MMU_WS2_UX        0x00000020
#define MMU_WS2_UW        0x00000010
#define MMU_WS2_UR        0x00000008
#define MMU_WS2_SX        0x00000004
#define MMU_WS2_SW        0x00000002
#define MMU_WS2_SR        0x00000001

/*-----------------------------------------------------------------------------+
| Cache control defines
+-----------------------------------------------------------------------------*/
#define DVLIM_TFLOOR_ENCODE(x)   (((x)&0x3F)<<22)
#define DVLIM_TFLOOR_DECODE(x)   (((x)>>22)&0x3F)
#define DVLIM_TCEILING_ENCODE(x) (((x)&0x3F)<<11)
#define DVLIM_TCEILING_DECODE(x) (((x)>>11)&0x3F)
#define DVLIM_NFLOOR_ENCODE(x)   (((x)&0x3F)<<0)
#define DVLIM_NFLOOR_DECODE(x)   (((x)>>0)&0x3F)

#define IVLIM_TFLOOR_ENCODE(x)   (((x)&0x3F)<<22)
#define IVLIM_TFLOOR_DECODE(x)   (((x)>>22)&0x3F)
#define IVLIM_TCEILING_ENCODE(x) (((x)&0x3F)<<11)
#define IVLIM_TCEILING_DECODE(x) (((x)>>11)&0x3F)
#define IVLIM_NFLOOR_ENCODE(x)   (((x)&0x3F)<<0)
#define IVLIM_NFLOOR_DECODE(x)   (((x)>>0)&0x3F)

#ifdef __cplusplus
}
#endif

#endif /* __A440CORE_INC__ */
