/* m4/a440gp.h, openbios_ebony, ebony_bios 1/12/00 12:46:13  
!------------------------------------------------------------------------------+
!
!       This source code has been made available to you by IBM on an AS-IS
!       basis.  Anyone receiving this source is licensed under IBM
!       copyrights to use it in any way he or she deems fit, including
!       copying it, modifying it, compiling it, and redistributing it either
!       with or without modifications.  No license under IBM patents or
!       patent applications is to be implied by the copyright license.
!
!       Any user of this software should understand that IBM cannot provide
!       technical support for this software and will not be responsible for
!       any consequences resulting from the use of this software.
!
!       Any person who transfers this source code or any derivative work
!       must include the IBM copyright notice, this paragraph, and the
!       preceding two paragraphs in the transferred software.
!
!       COPYRIGHT   I B M   CORPORATION 1995
!       LICENSED MATERIAL  -  PROGRAM PROPERTY OF I B M
!-------------------------------------------------------------------------------
!-------------------------------------------------------------------------------
!
!  File Name:   a440gp.h
!
!  Function:    Architecture defines for 440GP used in assembly language.
!
!  Author:      Alan Booker
!
!  Change Activity-
!
!  Date        Description of Change                                       BY
!  ---------   ---------------------                                       ---
!  9Mar95      Created							   ajb
! 12Jun96      Port to 401GF						   NL
!  7Aug96      Add brcr extended mnemonics 				   NL
! 12Sep96      Change BESR-related entries 				   NL
! 12Mar99      Port to 405GP               				   NL
! 07-Dec-99    Added OCM for 405GP pass2                                   JWB
! 08-Sep-00    Port to 440GP                                               DWG
!
!-------------------------------------------------------------------------------
*/

#ifndef __A440GP_INC__
#define __A440GP_INC__

#ifdef __cplusplus
extern "C" {
#endif

/*-----------------------------------------------------------------------------+
| 440GP Board and chip clocking parameters.
+-----------------------------------------------------------------------------*/
#define M_MIN                    6
#define M_MAX                   40
#define SYS_MIN           25000000
#define SYS_MAX           66666666
#define SYS_PERIOD_MIN       15000
#define SYS_PERIOD_MAX       40000
#define VCO_MIN          500000000
#define VCO_MAX         1333333333
#define CPU_MIN          100000000
#define CPU_MAX          600000000
#define CPU_PERIOD_MIN        1600
#define CPU_PERIOD_MAX       10000
#define PLB_MIN          100000000
#define PLB_MAX          133333333
#define PCI_MIN           33333333
#define PCI_MAX          133333333
#define OPB_MIN           22000000     /* OPB must be 2x serial clk */
#define OPB_MAX           66666666
#define EBC_MIN                  1
#define EBC_MAX           66666666

#define CPU_DEFAULT      400000000
#define PLB_DEFAULT      133333333
#define OPB_DEFAULT       66666666
#define EBC_DEFAULT       66666666

/* SPRs */
#include <openbios/a440core.h>

/* DCRs */
#define CPC0_DCR_BASE 0x0E0
#define POWERMAN_DCR_BASE 0x0B0 
#include <openbios/cpc.h>

#define DMA0_DCR_BASE 0x100
#include <openbios/dma.h>

#define EBC0_DCR_BASE 0x012
#include <openbios/ebc.h>

#define EMAC0_BASE 0xE0000800
#define EMAC1_BASE 0xE0000900
#include <openbios/emac.h>

#define IIC0_MMIO_BASE 0xE0000400
#define IIC1_MMIO_BASE 0xE0000500
#include <openbios/iic.h>

#define MAL0_DCR_BASE 0x180
#include <openbios/mal.h>

#define OPB0_DCR_BASE 0x0A0
#include <openbios/opb.h>

#define PCIX0_CFGADDR 0xDEC00000
#define PCIX0_CFGDATA 0xDEC00004
#define PCIX0_CFGBASE 0xDEC80000
#include <openbios/pcix.h>

#define PLB0_DCR_BASE 0x080
#include <openbios/plb.h>

#define SDRAM0_DCR_BASE 0x010 
#include <openbios/sdram.h>

#define SRAM0_DCR_BASE 0x020
#include <openbios/sram.h>

/* Specific MMIO Regs */
#define UART0_MMIO_BASE 0xE0000200
#define UART1_MMIO_BASE 0xE0000300
#include <openbios/uart.h>

#if 0
#define UIC0_DCR_BASE 0x0C0
#define UIC1_DCR_BASE 0x0D0
#include <openbios/uic.h>
#endif

#define ZMII0_BASE 0xE0000780
#include <openbios/zmii.h>

#ifdef __cplusplus
}
#endif

#endif /* __A440GP_INC__ */
