/* m4/dma.inc, openbios_ebony, ebony_bios 7/9/00 14:37:45 
!------------------------------------------------------------------------------+
!
!       This source code has been made available to you by IBM on an AS-IS
!       basis.  Anyone receiving this source is licensed under IBM
!       copyrights to use it in any way he or she deems fit, including
!       copying it, modifying it, compiling it, and redistributing it either
!       with or without modifications.  No license under IBM patents or
!       patent applications is to be implied by the copyright license.
!
!       Any user of this software should understand that IBM cannot provide
!       technical support for this software and will not be responsible for
!       any consequences resulting from the use of this software.
!
!       Any person who transfers this source code or any derivative work
!       must include the IBM copyright notice, this paragraph, and the
!       preceding two paragraphs in the transferred software.
!
!       COPYRIGHT   I B M   CORPORATION 1995
!       LICENSED MATERIAL  -  PROGRAM PROPERTY OF I B M
!-------------------------------------------------------------------------------
!-------------------------------------------------------------------------------
!
!  File Name:   dma.inc
!
!  Function:    Defines for DMA registers used in assembly language.
!
!  Author:      Neil Leeder
!
!  Change Activity-
!
!  Date        Description of Change                                       BY
!  ---------   ---------------------                                       ---
!  12-Apr-99   Created							   NL
!  05-May-00   New names                                                   NL
!  13-Sep-00   Port to 440GP                                               DWG
!
!-------------------------------------------------------------------------------
*/

#ifndef __DMA_INC__
#define __DMA_INC__

#ifdef __cplusplus
extern "C" {
#endif

#define DMA0_CR0  (DMA0_DCR_BASE+0x00)  /* DMA channel control register 0      */
#define DMA0_CT0  (DMA0_DCR_BASE+0x01)  /* DMA count register 0                */
#define DMA0_SAH0 (DMA0_DCR_BASE+0x02)  /* DMA source address high reg 0       */
#define DMA0_SAL0 (DMA0_DCR_BASE+0x03)  /* DMA source address low reg 0        */
#define DMA0_DAH0 (DMA0_DCR_BASE+0x04)  /* DMA destination address high reg 0  */
#define DMA0_DAL0 (DMA0_DCR_BASE+0x05)  /* DMA destination address low reg 0   */
#define DMA0_SGH0 (DMA0_DCR_BASE+0x06)  /* DMA scatter/gather desc high addr 0 */
#define DMA0_SGL0 (DMA0_DCR_BASE+0x07)  /* DMA scatter/gather desc low addr 0  */
#define DMA0_CR1  (DMA0_DCR_BASE+0x08)  /* DMA channel control register 1      */
#define DMA0_CT1  (DMA0_DCR_BASE+0x09)  /* DMA count register 1                */
#define DMA0_SAH1 (DMA0_DCR_BASE+0x0A)  /* DMA source address high reg 1       */
#define DMA0_SAL1 (DMA0_DCR_BASE+0x0B)  /* DMA source address low reg 1        */
#define DMA0_DAH1 (DMA0_DCR_BASE+0x0C)  /* DMA destination address high reg 1  */
#define DMA0_DAL1 (DMA0_DCR_BASE+0x0D)  /* DMA destination address low reg 1   */
#define DMA0_SGH1 (DMA0_DCR_BASE+0x0E)  /* DMA scatter/gather desc high addr 1 */
#define DMA0_SGL1 (DMA0_DCR_BASE+0x0F)  /* DMA scatter/gather desc low addr 1  */
#define DMA0_CR2  (DMA0_DCR_BASE+0x10)  /* DMA channel control register 2      */
#define DMA0_CT2  (DMA0_DCR_BASE+0x11)  /* DMA count register 2                */
#define DMA0_SAH2 (DMA0_DCR_BASE+0x12)  /* DMA source address high reg 2       */
#define DMA0_SAL2 (DMA0_DCR_BASE+0x13)  /* DMA source address low reg 2        */
#define DMA0_DAH2 (DMA0_DCR_BASE+0x14)  /* DMA destination address high reg 2  */
#define DMA0_DAL2 (DMA0_DCR_BASE+0x15)  /* DMA destination address low reg 2   */
#define DMA0_SGH2 (DMA0_DCR_BASE+0x16)  /* DMA scatter/gather desc high addr 2 */
#define DMA0_SGL2 (DMA0_DCR_BASE+0x17)  /* DMA scatter/gather desc low addr 2  */
#define DMA0_CR3  (DMA0_DCR_BASE+0x18)  /* DMA channel control register 3      */
#define DMA0_CT3  (DMA0_DCR_BASE+0x19)  /* DMA count register 3                */
#define DMA0_SAH3 (DMA0_DCR_BASE+0x1A)  /* DMA source address high reg 3       */
#define DMA0_SAL3 (DMA0_DCR_BASE+0x1B)  /* DMA source address low reg 3        */
#define DMA0_DAH3 (DMA0_DCR_BASE+0x1C)  /* DMA destination address high reg 3  */
#define DMA0_DAL3 (DMA0_DCR_BASE+0x1D)  /* DMA destination address low reg 3   */
#define DMA0_SGH3 (DMA0_DCR_BASE+0x1E)  /* DMA scatter/gather desc high addr 3 */
#define DMA0_SGL3 (DMA0_DCR_BASE+0x1F)  /* DMA scatter/gather desc low addr 3  */
#define DMA0_SR   (DMA0_DCR_BASE+0x20)  /* DMA status register                 */
#define DMA0_SGC  (DMA0_DCR_BASE+0x23)  /* DMA scatter/gather command register */
#define DMA0_SLP  (DMA0_DCR_BASE+0x25)  /* DMA sleep mode                      */
#define DMA0_POL  (DMA0_DCR_BASE+0x26)  /* DMA polarity                        */

/*-----------------------------------------------------------------------------+
| DMA Channel Control register
+-----------------------------------------------------------------------------*/
#define DMA_CR_CE                 0x80000000   /* Channel Enable             */
#define DMA_CR_CIE                0x40000000   /* Channel Interrupt Enable   */
#define DMA_CR_TD                 0x20000000   /* Transfer Direction         */
#define DMA_CR_PL                 0x10000000   /* Peripheral Location        */
#define DMA_CR_PW_MASK            0x0E000000   /* Peripheral width mask      */
#define DMA_CR_PW_8               0x00000000   /* 8 bit peripheral width     */
#define DMA_CR_PW_16              0x02000000   /* 16 bit peripheral width    */
#define DMA_CR_PW_32              0x04000000   /* 32 bit peripheral width    */
#define DMA_CR_PW_64              0x06000000   /* 64 bit peripheral width    */
#define DMA_CR_PW_128             0x08000000   /* 128 bit peripheral width   */
#define DMA_CR_DAI                0x01000000   /* Destination Addr Increment */
#define DMA_CR_SAI                0x00800000   /* Source Address Increment   */
#define DMA_CR_BEN                0x00400000   /* Buffer Enable              */
#define DMA_CR_TM_MASK            0x00300000   /* Transfer mode mask         */
#define DMA_CR_TM_BUFFERED        0x00000000   /* Peripheral                 */
#define DMA_CR_TM_SW_MEM_TO_MEM   0x00200000   /* Software started mem to mem*/
#define DMA_CR_TM_HW_MEM_TO_MEM   0x00300000   /* Hardware paced mem to mem  */
#define DMA_CR_PSC_MASK           0x000C0000   /* Peripheral Setup Cycles    */
#define DMA_CR_PSC_0              0x00000000   /* 0 Peripheral Setup Cycles  */
#define DMA_CR_PSC_1              0x00040000   /* 1 Peripheral Setup Cycles  */
#define DMA_CR_PSC_2              0x00080000   /* 2 Peripheral Setup Cycles  */
#define DMA_CR_PSC_3              0x000C0000   /* 3 Peripheral Setup Cycles  */
#define DMA_CR_PSC(n) (((n)&0x3)<<18)          /* n Peripheral setup cycles  */
#define DMA_CR_PWC(n) (((n)&0x3f)<<12)         /* n peripheral wait cycles   */
#define DMA_CR_PHC(n) (((n)&0x7)<<9)           /* n peripheral hold cycles   */
#define DMA_CR_ETD                0x00000100   /* EOT/TC Pin Direction       */
#define DMA_CR_TCE                0x00000080   /* Terminal Count Enable      */
#define DMA_CR_CP_MASK            0x00000060   /* Channel Priority           */
#define DMA_CR_CP_0               0x00000000   /*   Low                      */
#define DMA_CR_CP_1               0x00000020   /*   Medium Low               */
#define DMA_CR_CP_2               0x00000040   /*   Medium High              */
#define DMA_CR_CP_3               0x00000060   /*   High                     */
#define DMA_CR_PF_MASK            0x00000018   /* Memory read prefetch trans */
#define DMA_CR_PF_1               0x00000000   /*   Prefetch 1 dword         */
#define DMA_CR_PF_2               0x00000008   /*   Prefetch 2 dword         */
#define DMA_CR_PF_4               0x00000010   /*   Prefetch 4 dword         */
#define DMA_CR_DEC                0x00000004   /* Address decrement          */
#define DMA_CR_SL                 0x00000002   /* Src Addr Location/Mem Loc  */

/*-----------------------------------------------------------------------------+
| DMA Status Register
+-----------------------------------------------------------------------------*/
#define DMA_SR_CS0                0x80000000   /* Ch 0: terminal count status*/
#define DMA_SR_CS1                0x40000000   /* Ch 1: terminal count status*/
#define DMA_SR_CS2                0x20000000   /* Ch 2: terminal count status*/
#define DMA_SR_CS3                0x10000000   /* Ch 3: terminal count status*/
#define DMA_SR_TS0                0x08000000   /* Ch 0: End of Transfr status*/
#define DMA_SR_TS1                0x04000000   /* Ch 1: End of Transfr status*/
#define DMA_SR_TS2                0x02000000   /* Ch 2: End of Transfr status*/
#define DMA_SR_TS3                0x01000000   /* Ch 3: End of Transfr status*/
#define DMA_SR_RI0                0x00800000   /* Ch 0: Error Status         */
#define DMA_SR_RI1                0x00400000   /* Ch 1: Error Status         */
#define DMA_SR_RI2                0x00200000   /* Ch 2: Error Status         */
#define DMA_SR_RI3                0x00100000   /* Ch 3: Error Status         */
#define DMA_SR_IR0                0x00080000   /* Ch 0: Internal DMA request */
#define DMA_SR_IR1                0x00040000   /* Ch 1: Internal DMA request */
#define DMA_SR_IR2                0x00020000   /* Ch 2: Internal DMA request */
#define DMA_SR_IR3                0x00010000   /* Ch 3: Internal DMA request */
#define DMA_SR_ER0                0x00008000   /* Ch 0: External DMA request */
#define DMA_SR_ER1                0x00004000   /* Ch 1: External DMA request */
#define DMA_SR_ER2                0x00002000   /* Ch 2: External DMA request */
#define DMA_SR_ER3                0x00001000   /* Ch 3: External DMA request */
#define DMA_SR_CB0                0x00000800   /* Ch 0: Channel Busy         */
#define DMA_SR_CB1                0x00000400   /* Ch 1: Channel Busy         */
#define DMA_SR_CB2                0x00000200   /* Ch 2: Channel Busy         */
#define DMA_SR_CB3                0x00000100   /* Ch 3: Channel Busy         */
#define DMA_SR_SG0                0x00000080   /* Ch 0: Scatter/gather status*/
#define DMA_SR_SG1                0x00000040   /* Ch 1: Scatter/gather status*/
#define DMA_SR_SG2                0x00000020   /* Ch 2: Scatter/gather status*/
#define DMA_SR_SG3                0x00000010   /* Ch 3: Scatter/gather status*/
#define DMA_SR_ALL0 (DMA0_SR_CS0|DMA0_SR_TS0|DMA0_SR_RI0|DMA0_SR_IR0|DMA0_SR_ER0|DMA0_SR_CB0|DMA0_SR_SG0)
#define DMA_SR_ALL1 (DMA0_SR_CS1|DMA0_SR_TS1|DMA0_SR_RI1|DMA0_SR_IR1|DMA0_SR_ER1|DMA0_SR_CB1|DMA0_SR_SG1)
#define DMA_SR_ALL2 (DMA0_SR_CS2|DMA0_SR_TS2|DMA0_SR_RI2|DMA0_SR_IR2|DMA0_SR_ER2|DMA0_SR_CB2|DMA0_SR_SG2)
#define DMA_SR_ALL3 (DMA0_SR_CS3|DMA0_SR_TS3|DMA0_SR_RI3|DMA0_SR_IR3|DMA0_SR_ER3|DMA0_SR_CB3|DMA0_SR_SG3)

/*-----------------------------------------------------------------------------+
| DMA Scatter/Gather Command Register
+-----------------------------------------------------------------------------*/
#define DMA_SGC_SSG0 0x80000000
#define DMA_SGC_SSG1 0x40000000
#define DMA_SGC_SSG2 0x20000000
#define DMA_SGC_SSG3 0x10000000

#ifdef __cplusplus
}
#endif

#endif /* __DMA_INC__ */
